/**
 * Ford-Fulkerson
 * 
 * Complexity (time): O(Ef)
 * Complexity (space): O(V^2)
 */

int N;
int par[MAX];
int graph[MAX][MAX], rg[MAX][MAX];

bool cont[MAX];

// Finds if there's a path between s and t using non-full
// residual edges
bool dfs(int s, int t) {
  cont[s] = true;
  if (s == t)
    return true;

  for (int i = 0; i < N; ++i)
    if (!cont[i] && rg[s][i]) {
      par[i] = s;

      if (dfs(i, t)) 
        return true;
    }

  return false;
}


// Returns maximum flow between s (source) and t (sink)
int ford_fulkerson(int s, int t) {
  int ans = 0;
  par[s] = -1;

  mset(cont, 0);
  memcpy(rg, graph, sizeof(graph));

  // Repeat while there's a valid path between s and t
  while (dfs(s, t)) {
    int flow = inf;

    // Get the minimum capacity among all edges of the chosen path
    for (int i = t; par[i] != -1; i = par[i])
      flow = min(flow, rg[par[i]][i]);

    // Update residual graph
    for (int i = t; par[i] != -1; i = par[i]) {
      rg[par[i]][i] -= flow;
      rg[i][par[i]] += flow;
    }

    ans += flow;
    mset(cont, 0);
  }

  return ans;
}
